setwd(".")


# Informujemy R o funkcjach w plikach zrodlowych
#####################################################

rm(list = ls())
source("codesModels.R")
source("codesStats.R")


# 1. Wgrywamy dane tygodniowe dla stp procentowych o rznych terminach zapadalnosci
################################################################################

filename <- "dataUS.csv"
data     <- read.csv(filename, header=TRUE, sep=";", dec=".", row.names="daty")


# Uwaga!!!
# w pliku US s stopy o ciaglej kapitalizacji (r*)
# w pliku UK, JP, DE s stopy proste          (r)
# r* = log(1+r/100)

daty     <- as.Date(rownames(data))

YC <- data[,1:9]    # data for yield curves
MV <- data[,10:11]  # data for macro variables

nazwyYC <- colnames(YC)
nazwyMV <- colnames(MV)

tau <- c(3,6,12,24,36,60,84,120,360) # okres zapadalnosci w miesiacach

N <- ncol(YC)						# liczba terminow zapadalnosci
T <- nrow(YC)						# liczba obserwacji
K <- length(tau)

# 2. Estymacja czynnikow modelu Nelsona-Siegla
######################################################################################
lam <- 0.0609;

LT  <- rep(1,K)			                                # czesc dlugookresowa
MT  <- (1-exp(-lam*tau))/(lam*tau) - exp(-lam*tau); # czesc sredniookresowa
ST  <- (1-exp(-lam*tau))/(lam*tau)				  # czesc krotkookresowa

x   <- cbind(LT,MT,ST)

beta <- solve(t(x)%*%x)%*%t(x)%*%t(YC)
beta <- t(beta)
L    <- beta[,1] # level factor
C    <- beta[,2] # curvature factor
S    <- beta[,3] # slope factor


# 3. Prognoza ex-post
#####################

# ustawienia, jezelicwiczenie dla podprboby
T0   <- 1                  # pocztek proby
T2   <- nrow(YC)	         # koniec proby
daty <- daty[T0:T2]
YC   <- YC[T0:T2,]
MV   <- MV[T0:T2,]
beta <- beta[T0:T2,]
T    <- T2-T0+1            # liczba obserwacji

start.date <- "1999-12-31"                           # data dla pierwszej prognozy
T1         <- as.numeric(which(daty == start.date))  # obserwacja dla pierwszej prognozy
horiz      <- 24

Mat     <- 3                                         # ktora stopa (Maturity in month)
yobs  <- YC[,which(tau==Mat)]                        # szereg dla danej stopy


# Benchmark (Nelson-Siegel)
##############
fctNS    <- NSfct(beta, daty, horiz, T1, Mat)
act      <- FctErr(yobs, fctNS)$act
errNS    <- FctErr(yobs, fctNS)$err
statNS   <- FctErrStats(errNS)
dmNS     <- matrix(NA,1,horiz)

# Random Walk
##############
fctRW    <- RWfct(yobs, daty, horiz, T1)
act      <- FctErr(yobs, fctRW)$act
errRW    <- FctErr(yobs, fctRW)$err
statRW   <- FctErrStats(errRW)
dmRW     <- DMstat(errRW, errNS, type="two.sided", power=2)


# ARMA(2,0) model
#############
fctARMA    <- ARMAfct(yobs, daty, horiz, T1, p=2, q=0)
act        <- FctErr(yobs, fctARMA)$act
errARMA    <- FctErr(yobs, fctARMA)$err
statARMA   <- FctErrStats(errARMA)
dmARMA     <- DMstat(errARMA, errNS, type="two.sided", power=2)

# DL-ARMA(2,0) model
#############
fctDLARMA    <- DLARMAfct(beta, daty, horiz, T1, Mat, p=2, q=0)
act          <- FctErr(yobs, fctDLARMA)$act
errDLARMA    <- FctErr(yobs, fctDLARMA)$err
statDLARMA   <- FctErrStats(errDLARMA)
dmDLARMA     <- DMstat(errDLARMA, errNS, type="two.sided", power=2)

# DL-VAR(2) model
#############
fctDLVAR    <- DLVARfct(beta, daty, horiz, T1, Mat, p=2)
act         <- FctErr(yobs, fctDLVAR)$act
errDLVAR    <- FctErr(yobs, fctDLVAR)$err
statDLVAR   <- FctErrStats(errDLVAR)
dmDLVAR     <- DMstat(errDLVAR, errNS, type="two.sided", power=2)


# Tables for point forecast
###########################

models <- c("NS", "RW", "ARMA", "DLARMA", "DLVAR")

ME   <- rbind(statNS$ME,statRW$ME,statARMA$ME,statDLARMA$ME,statDLVAR$ME)
MAE  <- rbind(statNS$MAE,statRW$MAE,statARMA$MAE,statDLARMA$MAE,statDLVAR$MAE)
RMSE <- rbind(statNS$RMSE,statRW$RMSE,statARMA$RMSE,statDLARMA$RMSE,statDLVAR$RMSE)
DM   <- rbind(dmNS,dmRW,dmARMA,dmDLARMA,dmDLVAR)

rownames(ME)   <- rownames(MAE) <- rownames(RMSE) <- rownames(DM) <- models; 
colnames(ME)   <- colnames(MAE) <- colnames(RMSE) <- colnames(DM) <- paste0("h=",1:horiz)


Hsel = c(1,2,3,6,9,12,18,24)


options(digits=3)
ME[,Hsel]
MAE[,Hsel]
RMSE[,Hsel]
DM[,Hsel]

# Test efektywnosci
###################################

h     = 1                           # horyzont
fct   = fctNS                        # wybierz model

ylevel  <- act[1:(T-T1-h+1),h] 
xlevel  <- fct[1:(T-T1-h+1),h] 
ychange <- act[1:(T-T1-h+1),h] - yobs[T1:(T-h)]
xchange <- fct[1:(T-T1-h+1),h] - yobs[T1:(T-h)]

par(mfrow=c(1,2), cex = 0.7, bty="l")
plot(xlevel,ylevel, main="poziom", xlab="prognoza", ylab="realizacja")
plot(xchange,ychange, main="zmiana", xlab="prognoza", ylab="realizacja")

fit <- lm(ylevel ~ xlevel)
summary(fit)
fit <- lm(ychange ~ xchange)
summary(fit)

require(car)
linearHypothesis(fit, hypothesis.matrix = diag(1,2), 
                 rhs=c(0,1), test="F") 

# Wykres prognoz rekursywnych
#####################################

fct   = fctNS                        # wybierz model
horiz = 24
plot(daty, yobs, main = "", xlab="", ylab="", type="l", lty=1, xlim=as.Date(c("1990-01-01","2018-12-31")))
for (n in 0:(T-T1)) {
  lines(daty[(T1+n+1):(T1+n+horiz)],fct[n+1,], type="l", lty=2, col="red")
}

# Writting to Excel
###############################

require(xlsx)
write.xlsx(ME[,Hsel], "results.xlsx", "ME", append=FALSE)
write.xlsx(MAE[,Hsel], "results.xlsx", "MAE", append=TRUE)
write.xlsx(RMSE[,Hsel], "results.xlsx", "RMSE", append=TRUE)
write.xlsx(DM[,Hsel], "results.xlsx", "DM", append=TRUE)

# if xlsx package not installed
write.csv(ME[,Hsel], file = "ME.csv")
write.csv(MAE[,Hsel], file = "MAE.csv")
write.csv(RMSE[,Hsel], file = "RMSE.csv")
write.csv(DM[,Hsel], file = "DM.csv")
